﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace NesCore.Video
{
    public class Palette
    {
        public Palette(byte tint)
        {
            tint &= 0x07;
            colours = new Colour[baseColourValues.Length];
            for (int index = 0; index < baseColourValues.Length; index++)
            {
                ulong colourValue = baseColourValues[index];

                byte red = (byte)(colourValue >> 16);
                byte green = (byte)(colourValue >> 8);
                byte blue = (byte)colourValue;

                // red tint
                if ((tint & 0x01) != 0)
                {
                    green = (byte)(green * 0.8);
                    blue = (byte)(blue * 0.8);
                }

                // green tint
                if ((tint & 0x02) != 0)
                {
                    red = (byte)(red * 0.8);
                    blue = (byte)(blue * 0.8);
                }

                // blue tint
                if ((tint & 0x04) != 0)
                {
                    red = (byte)(red * 0.8);
                    green = (byte)(green * 0.8);
                }

                colours[index] = new Colour(red, green, blue);
            }
        }


        public Colour this[byte index]
        {
            get { return colours[index % colours.Length];  }
        }

        private static readonly ulong[] baseColourValues =
        {
            0x666666, 0x002A88, 0x1412A7, 0x3B00A4, 0x5C007E, 0x6E0040, 0x6C0600, 0x561D00,
            0x333500, 0x0B4800, 0x005200, 0x004F08, 0x00404D, 0x000000, 0x000000, 0x000000,
            0xADADAD, 0x155FD9, 0x4240FF, 0x7527FE, 0xA01ACC, 0xB71E7B, 0xB53120, 0x994E00,
            0x6B6D00, 0x388700, 0x0C9300, 0x008F32, 0x007C8D, 0x000000, 0x000000, 0x000000,
            0xFFFEFF, 0x64B0FF, 0x9290FF, 0xC676FF, 0xF36AFF, 0xFE6ECC, 0xFE8170, 0xEA9E22,
            0xBCBE00, 0x88D800, 0x5CE430, 0x45E082, 0x48CDDE, 0x4F4F4F, 0x000000, 0x000000,
            0xFFFEFF, 0xC0DFFF, 0xD3D2FF, 0xE8C8FF, 0xFBC2FF, 0xFEC4EA, 0xFECCC5, 0xF7D8A5,
            0xE4E594, 0xCFEF96, 0xBDF4AB, 0xB3F3CC, 0xB5EBF2, 0xB8B8B8, 0x000000, 0x000000,
        };

        private Colour[] colours;
    }
}
