﻿using CPF;
using CPF.Animation;
using CPF.Controls;
using CPF.Drawing;
using CPF.Input;
using CPF.Shapes;
using CPF.Styling;
using NesCore.Storage;
using NesCore.Utility;
using NesCore.Video;
using SharpNes;
using SharpNes.Database;
using SharpNes.Input;
using SharpNes.Settings;
using SharpNes.Audio;
using System;
using System.Collections.Generic;
using System.IO;
using System.IO.Compression;
using System.Linq;
using System.Text;

namespace NESPlayer
{
    public class NesVideoPanel : Control
    {
        protected override void InitializeComponent()
        {

        }
        protected async override void OnInitialized()
        {
            base.OnInitialized();
            this.Focusable = true;
            if (DesignMode)
            {
                return;
            }
            this.keyboardState = new KeyboardState();

            // map for mouse states to help controller mapping
            this.mouseState = new MouseState();

            // set up joystick object before console
            this.gameControllerManager = new GameControllerManager();
            Console = new NesCore.Console();

            Console.Processor.Lockup = () =>
            {
                Console.Halt();
                OnGameStop(this, EventArgs.Empty);
                MessageBox.Show("KIL instruction enountered at address " +
                    Hex.Format(Console.Processor.State.ProgramCounter) +
                    ". There may be a problem with the ROM or a software bug in the emulator. ROM Details: " + cartridge.ToString(),
                    "Processor Error");
            };

            bitmapBuffer = new FastBitmap(256, 240);

            Console.Video.WritePixel = WritePixel;

            Console.Video.ShowFrame = () =>
            {
                Invalidate();

                // frame rate
                DateTime currentDateTime = DateTime.Now;
                double currentDeltaTime = (currentDateTime - frameDateTime).TotalSeconds;
                frameDateTime = currentDateTime;
                averageDeltaTime = averageDeltaTime * 0.9 + currentDeltaTime * 0.1;
                int frameRate = (int)(1.0 / averageDeltaTime);
                //frameRateStatusLabel.Text = frameRate + " FPS";

                //// MP4 recording
                //if (videoRecordingMp4)
                //    mp4VideoEncoder.WriteVideoFrame(bitmapBuffer.Bitmap);

                //// GIF recording
                //if (videoRecordingGif)
                //    gifVideoEncoder.AddFrame(bitmapBuffer.Bitmap, 0, 0, TimeSpan.FromMilliseconds(16));

            };


            Console.Audio.SampleRate = 44100;

            float[] outputBuffer = new float[256];
            int writeIndex = 0;

            apuAudioProvider = new ApuAudioProvider();

            Console.Audio.WriteSample = (sampleValue) =>
            {
                // fill buffer
                outputBuffer[writeIndex++] = sampleValue;
                writeIndex %= outputBuffer.Length;

                // when buffer full, send to wave provider
                if (writeIndex == 0)
                    apuAudioProvider.Queue(outputBuffer);
            };

            //waveOut = new WaveOut();
            //waveOut.DesiredLatency = 100;

            //waveOut.Init(apuAudioProvider);




            // load input settings
            InputSettings inputSettings = null;

            // if never set, create and save
            if (inputSettings == null)
            {
                inputSettings = new InputSettings();
                //Properties.Settings.Default.InputSettings = inputSettings;
                //Properties.Settings.Default.Save();
            }

            // if no joypad set, configure default keyboard controls
            if (inputSettings.Joypads.Count == 0)
            {
                inputSettings.BuildDefaultSettings();
                //Properties.Settings.Default.InputSettings = inputSettings;
                //Properties.Settings.Default.Save();
            }
            JoypadSetting1 = inputSettings.Joypads[0];

            // assign joypads
            foreach (JoypadSettings joyPadSettings in inputSettings.Joypads)
                Console.ConnectController(joyPadSettings.Port,
                    joyPadSettings.ConfigureJoypad(keyboardState, gameControllerManager));

            // assign zappers
            //foreach (ZapperSettings zapperSettings in inputSettings.Zappers)
            //    Console.ConnectController(zapperSettings.Port,
            //        zapperSettings.ConfigureZapper(mouseState));



            nstDatabase = new NstDatabase();

            Cartridge.DetermineMapperId = DetermineCartridgeMapperId;
            timer = new CPF.Threading.DispatcherTimer { Interval = TimeSpan.FromMilliseconds(10), IsEnabled = true };
            timer.Tick += Timer_Tick;

            FilterPause = await ResourceManager.GetImage("res://NESPlayer/FilterPause.png");
        }

        void WritePixel(in byte x, in byte y, in Colour colour, in PixelType pixelType)
        {
            // do not render top and bottom 8 rows
            var c = colour;
            if (y < 8 || y > 231)
                c = Colour.Black;

            int offset = (y * 256 + x) * 4;

            //if (motionBlur)
            //{
            //    float delta = (colour.Blue - bitmapBuffer.Bits[offset]) / 2f;
            //    if (Math.Abs(delta) < 1) delta = Math.Sign(delta);
            //    bitmapBuffer.Bits[offset++] += (byte)delta;
            //    delta = (colour.Green - bitmapBuffer.Bits[offset]) / 2f;
            //    if (Math.Abs(delta) < 1) delta = Math.Sign(delta);
            //    bitmapBuffer.Bits[offset++] += (byte)delta;
            //    delta = (colour.Red - bitmapBuffer.Bits[offset]) / 2f;
            //    if (Math.Abs(delta) < 1) delta = Math.Sign(delta);
            //    bitmapBuffer.Bits[offset] += (byte)delta;
            //}
            //else
            //{
            bitmapBuffer.Bits[offset++] = c.Blue;
            bitmapBuffer.Bits[offset++] = c.Green;
            bitmapBuffer.Bits[offset] = c.Red;
            //}
        }
        Image FilterPause;

        public JoypadSettings JoypadSetting1 { get; set; }

        public void SaveSetting()
        {
            Console.ConnectController(JoypadSetting1.Port, JoypadSetting1.ConfigureJoypad(keyboardState, gameControllerManager));
        }

        private void Timer_Tick(object sender, EventArgs e)
        {
            OnGameTick(sender, e);
        }

        CPF.Threading.DispatcherTimer timer;
        public NesCore.Console Console { get; private set; }

        private Cartridge cartridge;
        private string cartridgeRomFilename;
        private string cartridgeSaveRamFilename;
        private bool cartridgeUsesGun;

        // execution state
        private GameState gameState;

        // video system
        private FastBitmap bitmapBuffer;
        private DateTime gameTickDateTime;

        //private Image resizedScreenFilter;
        //private Font resizedCaptionFont;

        // frame rate handling
        private DateTime frameDateTime;
        private double averageDeltaTime;

        // audio system
        //private WaveOut waveOut;
        private ApuAudioProvider apuAudioProvider;

        // input system
        private KeyboardState keyboardState;
        private MouseState mouseState;
        private GameControllerManager gameControllerManager;
        // NST database
        private NstDatabase nstDatabase;


        private byte DetermineCartridgeMapperId(uint romCrc, byte romMapperId)
        {
            NstDatabaseEntry nstDatabaseEntry = nstDatabase[romCrc];

            if (nstDatabaseEntry == null)
                return romMapperId;

            //if (nstDatabaseEntry.MapperId != romMapperId)
            //    statusHistoryLabel.Text = "Incorrect Mapper ID detected (" + romMapperId + "), should be " + nstDatabaseEntry.MapperId;

            return nstDatabaseEntry.MapperId;
        }

        private string ReplaceRomExtension(string cartridgeRomPath, string extension)
        {
            if (!extension.StartsWith("."))
                extension = "." + extension;
            return cartridgeRomPath.Replace(".nes", extension).Replace(".zip", extension);
        }


        public bool LoadCartridgeRom(string cartridgeRomPath)
        {
            try
            {
                // save previous SRAM if applicable
                if (this.cartridge != null && this.cartridge.SaveRam.Modified)
                    StoreSaveRam();

                Stream cartridgeRomStream = GetCartridgeRomStream(cartridgeRomPath);

                BinaryReader romBinaryReader = new BinaryReader(cartridgeRomStream);
                Cartridge newCartridge = new Cartridge(romBinaryReader);
                romBinaryReader.Close();

                // load SRAM if file exists
                this.cartridgeSaveRamFilename = ReplaceRomExtension(cartridgeRomPath, ".sram");

                if (File.Exists(this.cartridgeSaveRamFilename))
                {
                    BinaryReader saveRamBinaryReader = new BinaryReader(new FileStream(this.cartridgeSaveRamFilename, FileMode.Open));
                    newCartridge.SaveRam.Load(saveRamBinaryReader);
                    saveRamBinaryReader.Close();
                    //statusHistoryLabel.Text = "SaveRam file detected and loaded";
                }

                Console.LoadCartridge(newCartridge);

                this.cartridge = newCartridge;

                this.cartridgeRomFilename = System.IO.Path.GetFileNameWithoutExtension(cartridgeRomPath);

                //this.Text = cartridgeRomFilename + " - " + Application.ProductName;

                // enable some menu items dependent on a loaded cartridge
                //filePropertiesMenuItem.Enabled = true;
                //gameMenuItem.Enabled = true;
                //optionsCheatsMenuItem.Enabled = true;

                //if (traceEnabled)
                //{
                //    Console.Cartridge.Map.ProgramBankSwitch =
                //        (address, size) => codeDisassemblyForm.InvalidateMemoryRange(address, size);
                //}

                OnGameStop(this, EventArgs.Empty);
                OnGameRunPause(this, EventArgs.Empty);

                //recentFileManager.AddRecentFile(cartridgeRomPath);

                // set video file paths
                //this.videoPathMp4 = ReplaceRomExtension(cartridgeRomPath, ".mp4");
                //this.videoPathGif = ReplaceRomExtension(cartridgeRomPath, ".gif");

                // set cursor to crosshairs if it is a gun-based game
                cartridgeUsesGun = false;
                NstDatabaseEntry nstDatabaseEntry = nstDatabase[cartridge.Crc];
                if (nstDatabaseEntry != null && nstDatabaseEntry.Peripherals.Contains(Peripheral.Zapper))
                {
                    cartridgeUsesGun = true;
                }
                //videoPanel.Cursor = cartridgeUsesGun ? Cursors.Cross : Cursors.Default;

                // load cheat file if present
                //cheatSystem.Clear();
                string cheatFilePath = ReplaceRomExtension(cartridgeRomPath, ".cht");
                if (File.Exists(cheatFilePath))
                {
                    //statusHistoryLabel.Text = "Cheat file loaded";
                }

                return true;
            }
            catch (Exception exception)
            {
                MessageBox.Show("Unable to load cartridge rom. Reason: " + exception.Message, "Open Game ROM");
                return false;
            }
        }

        /// <summary>
        /// Tries to get a stream to a cartridge rom file using the given path.
        /// If the file is a ZIP archive, the a stream to the first NES file is
        /// returned.
        /// </summary>
        /// <param name="cartridgeRomPath">Path to NES or ZIP file</param>
        /// <returns></returns>
        private Stream GetCartridgeRomStream(string cartridgeRomPath)
        {
            Stream cartridgeRomStream = null;

#if !Net4
            if (System.IO.Path.GetExtension(cartridgeRomPath).ToLower() == ".zip")
            {
                Stream zipStream = new FileStream(cartridgeRomPath, FileMode.Open);
                ZipArchive zipArchive = new ZipArchive(zipStream, ZipArchiveMode.Read);

                // find first nes file
                foreach (ZipArchiveEntry entry in zipArchive.Entries)
                {
                    if (entry.FullName.EndsWith(".nes", StringComparison.OrdinalIgnoreCase))
                    {
                        cartridgeRomStream = entry.Open();
                        break;
                    }
                }
            }
#endif

            if (cartridgeRomStream == null)
                cartridgeRomStream = new FileStream(cartridgeRomPath, FileMode.Open);

            return cartridgeRomStream;
        }


        private void OnGameRunPause(object sender, EventArgs eventArgs)
        {
            switch (gameState)
            {
                case GameState.Stopped:
                    // run
                    Console.Reset();
                    gameTickDateTime = DateTime.Now;
                    averageDeltaTime = 1.0 / 60.0;
                    apuAudioProvider.Enabled = true;
                    gameState = GameState.Running;
                    //waveOut.Play();
                    break;
                case GameState.Running:
                    // pause
                    apuAudioProvider.Enabled = false;
                    gameState = GameState.Paused;
                    break;
                case GameState.Paused:
                    // resume;
                    gameTickDateTime = DateTime.Now;
                    averageDeltaTime = 1.0 / 60.0;
                    apuAudioProvider.Enabled = true;
                    gameState = GameState.Running;
                    break;
            }

            Invalidate();
            //UpdateGameMenuItems();
            //UpdateRecordMenuItems();
        }

        private void OnGameReset(object sender, EventArgs eventArgs)
        {
            OnGameStop(sender, eventArgs);
            Console.Reset();
            OnGameRunPause(sender, eventArgs);
        }

        private void OnGameStop(object sender, EventArgs eventArgs)
        {
            if (gameState == GameState.Stopped)
                return;

            Invalidate();
            gameState = GameState.Stopped;
            //waveOut.Stop();

            // save previous SRAM if applicable
            if (this.cartridge.SaveRam.Modified)
                StoreSaveRam();

            // stop recording if active
            //if (videoRecording)
            //    OnRecordVideoMp4StartStop(this, EventArgs.Empty);

            //UpdateGameMenuItems();
            //UpdateRecordMenuItems();
        }

        private void StoreSaveRam()
        {
            BinaryWriter saveRamBinaryWriter = new BinaryWriter(new FileStream(this.cartridgeSaveRamFilename, FileMode.OpenOrCreate, FileAccess.Write));
            this.cartridge.SaveRam.Save(saveRamBinaryWriter);
            saveRamBinaryWriter.Close();
            //statusHistoryLabel.Text = "SaveRam saved to disk";

        }
        protected override void OnKeyDown(KeyEventArgs e)
        {
            base.OnKeyDown(e);
            if (e.Key == Keys.Tab)
            {
                e.Handled = true;
            }
            keyboardState[e.Key] = true;
        }

        protected override void OnKeyUp(KeyEventArgs e)
        {
            base.OnKeyUp(e);
            keyboardState[e.Key] = false;
        }

        protected override void OnRender(DrawingContext dc)
        {
            base.OnRender(dc);
            if (bitmapBuffer != null)
            {
                dc.DrawImage(bitmapBuffer.Bitmap, new Rect(new Point(), ActualSize), new Rect(0, 0, bitmapBuffer.Bitmap.Width, bitmapBuffer.Bitmap.Height));
                if (gameState == GameState.Paused)
                {
                    var size = ActualSize;
                    dc.DrawImage(FilterPause, new Rect(new Point(), ActualSize), new Rect(0, 0, FilterPause.Width, FilterPause.Height));
                    var textSize = dc.DrawingFactory.MeasureString("暂停", new Font(FontFamily, 36, FontStyles.Italic));

                    var outlineThickness = size.Height / 120;
                    textSize.Width += outlineThickness;
                    textSize.Height += outlineThickness;

                    float textX = (size.Width - textSize.Width) / 2;
                    float textY = (size.Height - textSize.Height) / 2;

                    using (PathGeometry graphicsPath = new PathGeometry(new Font(this.FontFamily, 36, FontStyles.Italic), "暂停"))
                    using (SolidColorBrush outlinePen = new SolidColorBrush(Color.Black))
                    using (LinearGradientBrush linearGradientBrush = new LinearGradientBrush(new GradientStop[] { new GradientStop(Color.White, 0), new GradientStop(Color.LightSkyBlue, 1) }, new Point(), new Point(0, textSize.Height), Matrix.Identity))
                    {
                        var m = Matrix.Identity;
                        m.Translate(textX, textY);
                        graphicsPath.Transform(m);
                        dc.DrawPath(outlinePen, new Stroke(outlineThickness) { LineJoin = LineJoins.Round }, graphicsPath);
                        dc.FillPath(linearGradientBrush, graphicsPath);
                    }
                }
            }
        }

        private void OnGameTick(object sender, EventArgs eventArgs)
        {
            gameControllerManager.UpdateState();

            if (gameState != GameState.Running)
                return;

            //#if DEBUG
            //            Console.Run(0.020);
            //#else
            DateTime currentTickDateTime = DateTime.Now;
            double tickDelta = (currentTickDateTime - gameTickDateTime).TotalSeconds;
            gameTickDateTime = currentTickDateTime;

            Console.Run(tickDelta);
            //#endif
        }

        protected override void OnMouseDown(MouseButtonEventArgs e)
        {
            base.OnMouseDown(e);
            this.Focus();
            Root.SetIMEEnable(false);
            if (gameState == GameState.Paused)
                OnGameRunPause(this, EventArgs.Empty);
        }

        protected override void OnLostFocus(RoutedEventArgs e)
        {
            base.OnLostFocus(e);
            if (gameState == GameState.Running)
                OnGameRunPause(this, e);
        }

        protected override void Dispose(bool disposing)
        {
            apuAudioProvider.Dispose();
            base.Dispose(disposing);

            if (bitmapBuffer != null)
            {
                bitmapBuffer.Dispose();
                bitmapBuffer = null;
            }
            if (timer != null)
            {
                timer.Dispose();
            }
        }
    }
}
