﻿using CPF;
using CPF.Animation;
using CPF.Controls;
using CPF.Drawing;
using CPF.Input;
using CPF.Shapes;
using CPF.Styling;
using CPF.Svg;
using SharpNes.Settings;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace NESPlayer
{
    public class Window1 : Window
    {
        protected override void InitializeComponent()
        {
            Title = "Nes游戏";
            Width = 500;
            Height = 400;
            MinHeight = 100;
            MinWidth = 100;
            Background = null;
            CanResize = true;
            Children.Add(new WindowFrame(this, new Panel
            {
                Width = "100%",
                Height = "100%",
                Children =
                {
                    new StackPanel
                    {
                        Orientation = Orientation.Horizontal,
                        Children =
                        {
                            new Button
                            {
                                Width = 68.4f,
                                Height = "100%",
                                MarginLeft = 0f,
                                Content = "打开游戏",
                                Commands =
                                {
                                    {
                                        nameof(Button.Click),
                                        (s,e)=>
                                        {
                                            LoadFile();
                                        }
                                    }
                                }
                            },
                            new Button
                            {
                                Width = 68.4f,
                                Height = "100%",
                                MarginLeft = 0f,
                                Content = "控制",
                                Commands =
                                {
                                    {
                                        nameof(Button.Click),
                                        (s,e)=>
                                        {
                                            ShowSetting=!ShowSetting;
                                        }
                                    }
                                }
                            }
                        },
                        MarginTop = 0f,
                        MarginRight = 0f,
                        MarginLeft = 0f,
                        Height = 30f,
                    },
                    new NesVideoPanel
                    {
                        MarginBottom=0,
                        MarginLeft=0,
                        MarginTop=30,
                        MarginRight=0,
                        Name=nameof(nes),
                        PresenterFor=this,
                    },
                    new Panel
                    {
                        BorderStroke = "1,Solid",
                        BorderFill = "#878787",
                        Children =
                        {
                            new Panel
                            {
                                Background = "#696969",
                                Children =
                                {
                                    new TextBlock
                                    {
                                        Foreground = "#FFFFFF",
                                        MarginLeft = 10.7f,
                                        Text = "手柄1设置",
                                    },
                                    new SVG
                                    {
                                        Fill = "#FFFFFF",
                                        Stretch = Stretch.Uniform,
                                        Source = "<svg t=\"1608443834491\" class=\"icon\" viewBox=\"0 0 1024 1024\" version=\"1.1\" xmlns=\"http://www.w3.org/2000/svg\" p-id=\"3142\" width=\"64\" height=\"64\"><path d=\"M587.471 512.669l397.486-397.607c20.069-20.07 20.069-53.03 0-73.221l-2.19-2.19c-20.068-20.069-53.03-20.069-73.22 0L512.06 437.745 114.575 39.53c-20.19-20.07-53.152-20.07-73.22 0l-2.19 2.189c-20.556 20.069-20.556 53.03 0 73.221l397.607 397.729L39.165 910.155c-20.07 20.069-20.07 53.03 0 73.22l2.19 2.19c20.068 20.069 53.03 20.069 73.22 0l397.607-397.607L909.79 985.565c20.069 20.069 53.03 20.069 73.22 0l2.19-2.19c20.069-20.068 20.069-53.03 0-73.22L587.471 512.669z m0 0\" p-id=\"3143\"></path></svg>",
                                        MarginRight = 10.5f,
                                        Height = 17f,
                                        Width = 17.6f,
                                        Commands =
                                        {
                                            {
                                                nameof(UIElement.MouseDown),
                                                (s,e)=>
                                                {
                                                    ShowSetting=false;
                                                }
                                            }
                                        }
                                    },
                                },
                                Width = "100%",
                                Height=30.9f,
                                MarginTop=0f,
                            },
                            new TextBlock
                            {
                                MarginLeft = 30f,
                                MarginTop = 44.4f,
                                Foreground = "#FFFFFF",
                                Text = "开始",
                            },
                            new TextBlock
                            {
                                MarginLeft = 30f,
                                MarginTop = 73f,
                                Foreground = "#FFFFFF",
                                Text = "选择",
                            },
                            new TextBlock
                            {
                                MarginLeft = 30f,
                                MarginTop = 102.1f,
                                Foreground = "#FFFFFF",
                                Text = "A",
                            },
                            new TextBlock
                            {
                                MarginTop = 131.2f,
                                MarginLeft = 30f,
                                Foreground = "#FFFFFF",
                                Text = "B",
                            },
                            new TextBlock
                            {
                                MarginLeft = 30f,
                                MarginTop = 158.1f,
                                Foreground = "#FFFFFF",
                                Text = "上",
                            },
                            new TextBlock
                            {
                                MarginLeft = 30f,
                                MarginTop = 185f,
                                Foreground = "#FFFFFF",
                                Text = "下",
                            },
                            new TextBlock
                            {
                                MarginLeft = 30f,
                                MarginTop = 210.7f,
                                Foreground = "#FFFFFF",
                                Text = "左",
                            },
                            new TextBlock
                            {
                                MarginLeft = 30f,
                                MarginTop = 238.2f,
                                Foreground = "#FFFFFF",
                                Text = "右",
                            },
                            new ComboBox
                            {
                                MarginRight = 40f,
                                Width = 120.6f,
                                MarginTop = 42.5f,
                                DisplayMemberPath="Item1",
                                SelectedValuePath="Item2",
                                Bindings =
                                {
                                    {
                                        nameof(ComboBox.Items),
                                        nameof(KeyItems),
                                        this
                                    },
                                    {
                                        nameof(ComboBox.SelectedValue),
                                        nameof(JoypadSettings.Start),
                                        null,
                                        BindingMode.TwoWay,
                                        (string a)=>(Keys)Enum.Parse(typeof(Keys), a.Replace("key:", "")),
                                        (Keys a)=>
                                        {
                                            return "key:" + a;
                                        }
                                    }
                                }
                            },
                            new ComboBox
                            {
                                MarginRight = 40f,
                                MarginTop = 71.1f,
                                Width = 120.6f,
                                DisplayMemberPath="Item1",
                                SelectedValuePath="Item2",
                                Bindings =
                                {
                                    {
                                        nameof(ComboBox.Items),
                                        nameof(KeyItems),
                                        this
                                    },
                                    {
                                        nameof(ComboBox.SelectedValue),
                                        nameof(JoypadSettings.Select),
                                        null,
                                        BindingMode.TwoWay,
                                        (string a)=>(Keys)Enum.Parse(typeof(Keys), a.Replace("key:", "")),
                                        (Keys a)=>
                                        {
                                            return "key:" + a;
                                        }
                                    }
                                }
                            },
                            new ComboBox
                            {
                                MarginRight = 40f,
                                Width = 120.6f,
                                MarginTop = 99.7f,
                                DisplayMemberPath="Item1",
                                SelectedValuePath="Item2",
                                Bindings =
                                {
                                    {
                                        nameof(ComboBox.Items),
                                        nameof(KeyItems),
                                        this
                                    },
                                    {
                                        nameof(ComboBox.SelectedValue),
                                        nameof(JoypadSettings.A),
                                        null,
                                        BindingMode.TwoWay,
                                        (string a)=>(Keys)Enum.Parse(typeof(Keys), a.Replace("key:", "")),
                                        (Keys a)=>
                                        {
                                            return "key:" + a;
                                        }
                                    }
                                }
                            },
                            new ComboBox
                            {
                                MarginRight = 40f,
                                MarginTop = 128.8f,
                                Width = 120.6f,
                                DisplayMemberPath="Item1",
                                SelectedValuePath="Item2",
                                Bindings =
                                {
                                    {
                                        nameof(ComboBox.Items),
                                        nameof(KeyItems),
                                        this
                                    },
                                    {
                                        nameof(ComboBox.SelectedValue),
                                        nameof(JoypadSettings.B),
                                        null,
                                        BindingMode.TwoWay,
                                        (string a)=>(Keys)Enum.Parse(typeof(Keys), a.Replace("key:", "")),
                                        (Keys a)=>
                                        {
                                            return "key:" + a;
                                        }
                                    }
                                }
                            },
                            new ComboBox
                            {
                                MarginRight = 40f,
                                Width = 120.6f,
                                MarginTop = 155.7f,
                                DisplayMemberPath="Item1",
                                SelectedValuePath="Item2",
                                Bindings =
                                {
                                    {
                                        nameof(ComboBox.Items),
                                        nameof(KeyItems),
                                        this
                                    },
                                    {
                                        nameof(ComboBox.SelectedValue),
                                        nameof(JoypadSettings.Up),
                                        null,
                                        BindingMode.TwoWay,
                                        (string a)=>(Keys)Enum.Parse(typeof(Keys), a.Replace("key:", "")),
                                        (Keys a)=>
                                        {
                                            return "key:" + a;
                                        }
                                    }
                                }
                            },
                            new ComboBox
                            {
                                MarginRight = 40f,
                                MarginTop = 182.6f,
                                Width = 120.6f,
                                DisplayMemberPath="Item1",
                                SelectedValuePath="Item2",
                                Bindings =
                                {
                                    {
                                        nameof(ComboBox.Items),
                                        nameof(KeyItems),
                                        this
                                    },
                                    {
                                        nameof(ComboBox.SelectedValue),
                                        nameof(JoypadSettings.Down),
                                        null,
                                        BindingMode.TwoWay,
                                        (string a)=>(Keys)Enum.Parse(typeof(Keys), a.Replace("key:", "")),
                                        (Keys a)=>
                                        {
                                            return "key:" + a;
                                        }
                                    }
                                }
                            },
                            new ComboBox
                            {
                                MarginRight = 40f,
                                Width = 120.6f,
                                MarginTop = 210.7f,
                                DisplayMemberPath="Item1",
                                SelectedValuePath="Item2",
                                Bindings =
                                {
                                    {
                                        nameof(ComboBox.Items),
                                        nameof(KeyItems),
                                        this
                                    },
                                    {
                                        nameof(ComboBox.SelectedValue),
                                        nameof(JoypadSettings.Left),
                                        null,
                                        BindingMode.TwoWay,
                                        (string a)=>(Keys)Enum.Parse(typeof(Keys), a.Replace("key:", "")),
                                        (Keys a)=>
                                        {
                                            return "key:" + a;
                                        }
                                    }
                                }
                            },
                            new ComboBox
                            {
                                MarginRight = 40f,
                                MarginTop = 238.2f,
                                Width = 120.6f,
                                DisplayMemberPath="Item1",
                                SelectedValuePath="Item2",
                                Bindings =
                                {
                                    {
                                        nameof(ComboBox.Items),
                                        nameof(KeyItems),
                                        this
                                    },
                                    {
                                        nameof(ComboBox.SelectedValue),
                                        nameof(JoypadSettings.Right),
                                        null,
                                        BindingMode.TwoWay,
                                        (string a)=>(Keys)Enum.Parse(typeof(Keys), a.Replace("key:", "")),
                                        (Keys a)=>
                                        {
                                            return "key:" + a;
                                        }
                                    }
                                }
                            },
                            new Button
                            {
                                Height = 21.5f,
                                Width = 72.8f,
                                MarginTop = 269f,
                                Content = "保存",
                                Commands =
                                {
                                    {
                                        nameof(Button.Click),
                                        (s,e)=>
                                        {
                                            nes.SaveSetting();
                                            ShowSetting=false;
                                        }
                                    }
                                }
                            },
                        },
                        Background = "#00000066",
                        Height = 299.4f,
                        Width = 301.2f,
                        Bindings =
                        {
                            {
                                nameof(Visibility),
                                nameof(ShowSetting),
                                this,
                                BindingMode.OneWay,
                                (bool v)=>v? Visibility.Visible: Visibility.Collapsed
                            },
                            {
                                nameof(DataContext),
                                nameof(JoypadSetting),
                                this
                            }
                        }
                    }
                }
            })
            {
                MaximizeBox = true
            });
            LoadStyleFile("res://NESPlayer/Stylesheet1.css");
            //加载样式文件，文件需要设置为内嵌资源

            if (!DesignMode)//设计模式下不执行
            {
                
            }
        }
        NesVideoPanel nes;

        protected override void OnInitialized()
        {
            base.OnInitialized();
            var values = Enum.GetValues(typeof(Keys));
            KeyItems = new Collection<(string, Keys)>();
            for (int i = 0; i < values.Length; i++)
            {
                KeyItems.Add((values.GetValue(i).ToString(), (Keys)values.GetValue(i)));
            }
            nes = FindPresenterByName<NesVideoPanel>(nameof(nes));
            JoypadSetting = nes.JoypadSetting1;
        }

        async void LoadFile()
        {
            using (OpenFileDialog of = new OpenFileDialog())
            {
                var fs = await of.ShowAsync(this);
                if (fs != null && fs.Length > 0)
                {
                    nes.LoadCartridgeRom(fs[0]);
                }
            }
        }
        public JoypadSettings JoypadSetting
        {
            get { return GetValue<JoypadSettings>(); }
            set { SetValue(value); }
        }

        public bool ShowSetting
        {
            get { return GetValue<bool>(); }
            set { SetValue(value); }
        }

        public Collection<(string,Keys)> KeyItems
        {
            get { return GetValue<Collection<(string, Keys)>>(); }
            set { SetValue(value); }
        }

        protected override void OnGotFocus(GotFocusEventArgs e)
        {
            base.OnGotFocus(e);
            var v = nes;
            if (!v.IsFocused)
            {
                v.Focus();
            }
        }



    }
}
